@@ -20,6 +20,33 @@ def correctness(expected_outputs, outputs):
 TestRun = namedtuple('TestRun', ['input_lines', 'expected_output_lines', 
                                  'output_lines', 'error_lines', 'correctness'])
 
+class Agent():
+    """
+    Agent: represents a running program that can be interacted with
+    """
+
+    def __init__(self, program, process):
+        self.program = program
+        self.process = process
+        
+        self.program.stdout = ''
+        self.program.stderr = ''
+
+    def act(self, input_lines):
+        input = '\n'.join(input_lines).encode()
+        stdout, stderr = self.process.communicate(input)
+        stdout = stdout.decode()
+        stderr = stderr.decode()
+        self.program.stdout += stdout
+        self.program.stderr += stderr
+        return stdout.splitlines()
+    
+    def close(self):
+        self.process.kill()
+    
+    def __del__(self):
+        self.close()
+
 class Program():
     """
     Program object: represents a runnable program in some programming language
@@ -57,10 +84,22 @@ class Program():
         Use 'force=True' to skip the error check.
         Raw stdout and stderr data will always be stored in `program.stdout` and `program.stderr` attributes
         """
-        
-        self.stdout, self.stderr = self.language.run(self.workdir, self.name, input_lines)
-        assert force or not self.stderr, self.stderr
+
+        input = '\n'.join(input_lines).encode()
+        stdout, stderr = self.language.run(self.workdir, self.name, input)
+        assert force or not stderr, stderr
+
+        self.stdout = stdout.decode()
+        self.stderr = stderr.decode()
         return self.stdout.splitlines()
+    
+    def start(self):
+        """
+        Launch the program and get a process object to communicate with it interactively
+        """
+
+        process = self.language.Popen(self.workdir, self.name)
+        return Agent(self, process)
 
     def test(self, test_cases, force=True):
         """
